from sqlalchemy import util
from sqlalchemy.orm import MapperExtension, class_mapper, EXT_CONTINUE

from properties import one_to_one

__all__ = ['OneToOneMapperExtension']

class OneToOneMapperExtension(MapperExtension):
    """MapperExtension to proxy properties on one-to-one relations.

    This extension proxies access to all properties of the specified
    one-to-one relations without an intermediate layer. 
    
    The intended use case is to allow a type composed of multiple tables to
    be easily mapped and queried as if it were one table.

    """
    def __init__(self, *related_classes, **kwargs):
        self.related_classes = util.to_list(related_classes)
        self.property_prefix = kwargs.get('property_prefix', '_')
    
    def instrument_class(self, mapper, class_):
        # Iterate through the specified classes and create relations,
        # python properties, and mapper properties for all of them.
        for value_class in self.related_classes:
            value_mapper = class_mapper(value_class, compile=False)
            key = self.property_prefix + value_mapper.local_table.key
            # mapper.add_property isn't ready yet, so we use...
            mapper._init_properties[key] = one_to_one(value_class)
        return EXT_CONTINUE


"""Release information and release tools"""

import os
from textwrap import dedent

# Define pypi release information
NAME = "BrainFreeze"
VERSION = "0.1a1"
DESCRIPTION = dedent("""
    BrainFreeze is an SQLAlchemy plugin for proxying properties on
    one-to-one related objects.  Kind of like SQLAlchemy's AssociationProxy,
    but for one-to-one relations instead of many-to-many relations.
""")
AUTHOR = "Ian Charnas, Brian Beck"
AUTHOR_EMAIL = "ian.charnas@gmail.com, exogen@gmail.com"
URL = "http://brainfreeze-alchemy.googlecode.com/"
DOWNLOAD_URL = "http://code.google.com/p/brainfreeze-alchemy/downloads/list"

# Add support for `python setup.py googlecode_upload`
#
# Note: This is not a very generalized way to do this, clearly I should be 
# using user_options instead of relying on some constants
GOOGLECODE_PROJECT = "brainfreeze-alchemy"

from distutils import log
from distutils.core import Command
from distutils.errors import DistutilsOptionError
from brainfreeze.release.googlecode_upload import upload_find_auth
class googlecode_upload(Command):
    """Setuptools Command to upload packages to google code"""
    description = "upload packages to google code"
    
    user_options = []
    
    def initialize_options(self):
        pass
        
    def finalize_options(self):
        pass

    def run(self):
        if not self.distribution.dist_files:
            raise DistutilsOptionError(
                "No dist file created in earlier command"
            )
        for command, pyversion, filename in self.distribution.dist_files:
            self.upload_file(command, pyversion, filename)
            
    def upload_file(self, command, pyversion, filename):
        command_filetype_map = {
            'bdist_egg': ("Python Egg", ['Type-Package', 'OpSys-All']),
            'bdist_msi': (
                "Microsoft Installer Package", 
                ['Type-Package', 'OpSys-Windows']
            ),
            'bdist_rpm': ("RPM", ['Type-Package', 'OpSys-Linux']),
            'bdist_wininst': (
                "Auto-Installing Executable for Microsoft Windows", 
                ['Type-Installer', 'OpSys-Windows']
            ),
            'sdist': ("Source Distribution", ['Type-Source', 'OpSys-All'])
        }
        
        self.announce("Submitting %s to Google Code" % filename, log.INFO)
        try:
            status, reason, url = upload_find_auth(
                filename, # file to upload
                GOOGLECODE_PROJECT, # which project to upload file to?
                command_filetype_map[command][0], # summary of file
                command_filetype_map[command][1] # labels for file
            )
            if url:
                self.announce(
                    "Google Code upload successful to URL: %s" % url
                )
            else:
                self.announce(
                    "Google Code upload failed.  Server said %s (%s)" %
                    (reason, status)
                )
        except Exception, e:
            self.announce("Upload failed: %s" % str(e), log.ERROR)      



#!python
"""Setuptools-based installation and packaging module"""

# Ensure setuptools is available (download & install if necessary)
import ez_setup
ez_setup.use_setuptools()
from setuptools import setup, find_packages

# Get package information (and googlecode_upload Command class)
import os
execfile(os.path.join("brainfreeze", "release", "__init__.py"))

# Check to make sure required python is available
def check_python():
    if sys.version_info < (2,3):
        print "Python 2.3 or higher is required to run BrainFreeze."
        sys.exit(1)

# Load long description from README file
readme = os.path.normpath(__file__ + "/../README")
f = open(readme)
LONG_DESCRIPTION = f.read()

# Let setuptools work its magic!
setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url=URL,
    download_url=DOWNLOAD_URL,
	packages=find_packages(), # find python packages in this dir
	include_package_data=True, # include more than just the code
	cmdclass={'googlecode_upload': googlecode_upload},
	keywords=["brainfreeze", "sqlalchemy", "orm"],
    test_suite="brainfreeze.test.test_docs.test_suite",
    classifiers=[
        "Development Status :: 3 - Alpha",
    #     "Environment :: Console",
    #     "Environment :: Plugins",
         "Intended Audience :: Developers",
    #     "Intended Audience :: End Users/Desktop",
         "License :: OSI Approved :: BSD License",
         "Operating System :: OS Independent",
         "Programming Language :: Python",
         "Topic :: Software Development :: Libraries :: Python Modules",
    #     "Topic :: Software Development :: Version Control",
    #     "Topic :: System :: Archiving",
    #     "Topic :: Utilities"
    ],
)


